/*
 * Copyright (c) 1996-1998 PFU Limited.
 */
package plugins;

import teikade.sys.CodePath;
import teikade.sys.PluginCommand;
import teikade.gui.component.DFrame;
import teikade.gui.Transcript;

/*
import inspect.Inspector;
import inspect.InspectorView;
import inspect.ReflectionInspector;
 */

import java.awt.PopupMenu;
import java.awt.MenuItem;
import java.awt.Component;
import java.awt.Container;
import java.awt.TextComponent;
import java.awt.TextArea;
import java.awt.Panel;
import java.awt.BorderLayout;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;

import pnuts.lang.Pnuts;
import pnuts.lang.Context;
import pnuts.lang.Package;
import pnuts.lang.PnutsException;

/**
 * The Pnuts based Workspace PluginCommand implementation.
 *
 * @author SATOH Osamu
 * @version 1.0
 * @date August 31, 1999
 */
public class PnutsWorkspacePlugin implements PluginCommand {
    public void initialize(String name) {
	Transcript.getTranscript().message(name + ": Pnuts Workspace Plugin\n");
    }

    public void invoke(String name, Object frame, CodePath cp) {
	DFrame f = new DFrame("Pnuts Workspace Plugin");
	f.add(new PnutsWorkspaceView(), BorderLayout.CENTER);
	f.setSize(400, 300);
	f.show();
    }

    public boolean isInvocableFor(String name, Object frame, CodePath cp) {
	return true;
    }

/*
    public void open(InspectorView view) {
	String title = "InspectorView on: " +
			    view.getModel().getTarget().toString();
	DFrame frame = new DFrame(title);
	frame.add(view, BorderLayout.CENTER);
	frame.show();
    }
 */

    class PnutsWorkspaceView extends Panel {
	private TextComponent text;
	private Context context;

	PnutsWorkspaceView() {
	    context = new Context(new Package());
	    context.set("this", this);
	    context.setErrorStream(null);
	    initComponents();
	}

	PnutsWorkspaceView(Object thisObject) {
	    context = new Context(new Package());
	    context.set("this", thisObject);
	    context.setErrorStream(null);
	    initComponents();
	}

	protected void initComponents() {
	    final PopupMenu popup = new PopupMenu("do it/print it");
	    popup.add(createDoItMenuItem());
	    //popup.add(createInspectMenuItem());
	    popup.add(createPrintItMenuItem());
	    text = createTextComponent();
	    text.addMouseListener(
		new MouseAdapter() {
		    public void mousePressed(MouseEvent e) {
			if (e.isPopupTrigger()) {
			    popup.show(text, e.getX(), e.getY());
			}
		    }
		}
	    );
	    text.add(popup);
	    final java.awt.Label label = new java.awt.Label("menu");
	    label.addMouseListener(
		new MouseAdapter() {
		    public void mousePressed(MouseEvent e) {
			popup.show(label, e.getX(), e.getY());
		    }
		}
	    );
	    label.add(popup);
	    setLayout(new BorderLayout());
	    add(text, BorderLayout.CENTER);
	    add(label, BorderLayout.SOUTH);
	}

	TextComponent createTextComponent() {
	    return new TextArea("", 20, 50, TextArea.SCROLLBARS_VERTICAL_ONLY);
	}

	MenuItem createDoItMenuItem() {
	    MenuItem item = new MenuItem("do it");
	    item.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent ev) {
		    try {
			String expr = text.getSelectedText();
			Pnuts.eval(expr, context);
		    } catch (PnutsException e) {
			insertAfter(e.getThrowable());
		    }
		}
	    });
	    return item;
	}

	MenuItem createPrintItMenuItem() {
	    MenuItem item = new MenuItem("print it");
	    item.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent ev) {
		    try {
			String expr = text.getSelectedText();
			Object result = Pnuts.eval(expr, context);
			insertAfter(result);
		    } catch (PnutsException e) {
			insertAfter(e.getThrowable());
		    }
		}
	    });
	    return item;
	}

/*
	MenuItem createInspectMenuItem() {
	    MenuItem item = new MenuItem("inspect");
	    item.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent ev) {
		    try {
			String expr = text.getSelectedText();
			Object result = Pnuts.eval(expr, context);
			Inspector model = ReflectionInspector.inspect(result);
			InspectorView view = new InspectorView(model);
			open(view);
		    } catch (PnutsException e) {
			insertAfter(e.getThrowable());
		    } catch (Exception e) {
			handleException(e, this);
		    }
		}
	    });
	    return item;
	}
 */

	protected void insertAfter(Object o) {
	    int start = text.getSelectionStart();
	    int end = text.getSelectionEnd();
	    String head = text.getText();
	    String tail = head.substring(end);
	    head = head.substring(0, end) + o;
	    text.setText(head + tail);
	    text.select(end, head.length());
	}
    }
}
