/*
 * Copyright (c) 1995-1998 PFU Limited.
 *	author Kazuhisa Shirakami
 */
package teikade.gui;

import teikade.sys.CodePath;
import teikade.sys.TeikadeException;
import teikade.util.Command;
import teikade.util.CommandListener;
import teikade.gui.menu.TeikadeMenuBar;
import teikade.gui.menu.CommandMenuBar;

public abstract class CommandFrame extends ListingFrame implements CommandListener, Runnable {
    private String name;
    private Command command;
    protected Thread commandThread = null;

    CommandFrame(CodePath cp, String title, String name) {
	super(cp, title);
	this.name = name;
    }

    protected TeikadeMenuBar createMenuBar() {
	return new CommandMenuBar();
    }

    public abstract void start();

    public abstract void restart();

    public void stop() {
	if (command != null) {
	    command.abort();
	    // maybe called aborted method
	}
    }

    protected void start(Command cmd) {
	if (cmd == null) return;
	try {
	    if (!select(-1)) return;
	} catch (TeikadeException e) {
	    information(e.getMessage());
	    return;
	}
	command = cmd;
	commandThread = new Thread(this,this.getClass().getName());
	commandThread.setPriority(Thread.MIN_PRIORITY);
	commandThread.start();
	((CommandMenuBar)getMenuBar()).getFileMenu().enable("stop");
    }

    public void run() {
	if (Thread.currentThread() == commandThread) {
	    try {
		command.execute();
	    } finally {
		commandThread = null;
	    }
	}
    }

    protected void restart(Command cmd) {
	stop();
	if (syncAndClearListItems()) {
	    start(cmd);
	}
    }

    public void executing(Command command) {
	addMessageWithDate(name + " started");
    }

    public void finished(Command command, int value) {
	end(" ended  ");
    }

    public void aborted(Command command, Exception e) {
	if (e == null) {
	    end(" aborted");
	} else {
	    end(" aborted: " + e.getMessage());
	}
    }

    private void end(String message) {
	command = null;
	commandThread = null;
	CommandMenuBar menuBar = (CommandMenuBar)getMenuBar();
	if (menuBar == null) return;	// frame was closed
	menuBar.getFileMenu().disable("stop");
	addMessageWithDate(name + message);
	update();
    }

    public boolean close() {
	if (!super.close()) return false;
	stop();
	return true;
    }
}
