/*
 * Copyright (c) 1995 PFU Limited.
 *	author Osamu Satoh, Kazuhisa Shirakami
 */
package teikade.gui;

import teikade.sys.*;
import teikade.util.*;
import teikade.gui.component.*;
import teikade.gui.editor.EditData;
import java.awt.*;
import java.util.*;

public class GrepFrame extends ListingFrame implements GrepListener {
    private Label patternLabel;
    private TextField patternField;
    private Panel patternPane;
    Grep grep;

    static void openOn(CodePath cp, String pattern, String title)
    throws TeikadeException {
        GrepFrame grep = new GrepFrame(cp, pattern, title);
	grep.open();
	grep.start();
    }

    public GrepFrame(CodePath cp, String pattern, String title)
    throws TeikadeException {
	super(cp, title);
	initialize(cp, pattern, title);
    }

    protected void initialize(CodePath cp, String pattern, String title)
    throws TeikadeException {
	if (title != null) {
	    setTitle(title);
	} else {
	    setTitle("Occurences of: '" + pattern + "'");
	}
	patternField.setText(pattern);
    }

    protected Component createListingItems() {
	patternField = new DTextField();
	patternLabel = new DLabel("egrep pattern:");
	patternPane = new Panel();
	patternPane.setLayout(new BorderLayout());
	patternPane.add(WEST, patternLabel);
	patternPane.add(CENTER, patternField);
	Panel panel = new Panel();
	panel.setLayout(new BorderLayout());
	panel.add(NORTH, patternPane);
	panel.add(CENTER, super.createListingItems());
	return panel;
    }

    protected void showPattern(boolean show) {
	if (show) {
	    patternPane.show();
	} else {
	    patternPane.hide();
	}
	pack();
    }

    public boolean action(Event ev, Object arg) {
	if (ev.target == patternField) {
	    String pattern = patternField.getText();
	    if (pattern.length() > 0) {
		setTitle("Occurence of: '" + pattern + "'");
		start();
	    }
	    return true;
	} else {
	    boolean ret = super.action(ev, arg);
	    return ret;
	}
    }

    public void start() {
	syncAndClearListItems();
	stop();
	grep = Grep.getInstance(startup, patternField.getText(), this);
	patternLabel.setText(grep.getPatternLabel());
	grep.execute();
    }

    public void stop() {
	if (grep != null) {
	    grep.abort();
	}
    }

    public void executing(Command grep) {
	addMessageWithDate("grep started");
    }

    public void finished(Command grep, int status) {
	if (status == 0) {
	    addMessageWithDate("grep finished");
	} else {
	    addMessageWithDate("grep finished abnormaly");
	}
	grep = null;
    }

    public void aborted(Command grep, Exception e) {
	if (e == null) {
	    addMessageWithDate("grep aborted ");
	} else {
	    addMessageWithDate("grep aborted: " + e.getMessage());
	}
	grep = null;
    }

    public void result(Grep grep, CodeLocation loc) {
	addItem(loc);
    }

    public void message(Grep grep, String message) {
	addMessage(message);
    }

    protected ListingFrame copyCreate(CodePath cp) {
	try {
	    return new GrepFrame(cp, patternField.getText(), "Same elements for: " + cp.printString());
	} catch (TeikadeException e) {
	    information("fail to copy frame: " + e.getMessage());
	    return null;
	}
    }
}
