/*
 * Copyright (c) 1995-1998 PFU Limited.
 *	author Osamu Satoh, Kazuhisa Shirakami
 */
package teikade.gui;

import teikade.sys.CodePath;
import teikade.sys.ClassManager;
import teikade.sys.TeikadeException;
import java.util.Vector;
import java.util.Enumeration;
import java.awt.Event;
import java.awt.List;

import teikade.gui.menu.*;
import teikade.debug.*;
import teikade.lang.*;
import java.lang.*;

public class HierarchyBrowser extends Browser implements Runnable {

    int selection = -1;
    Vector codePath = new Vector();

    Thread hierarchyThread = null;

    private static final String PROPERTY_LIST_WIDTH_CLASS =
	"attribute.hierarching.list.width.class";

    AccessFlags getHideMethods() {
        return ((HierarchyMenuBar)getMenuBar()).hideMethods();
    }

    static Browser openOn(CodePath cp, CodePath select, String title) {
	HierarchyBrowser hb = new HierarchyBrowser(cp, select, title);
	hb.open();
	hb.start();
	return hb;
    }

    protected void start() {
	hierarchyThread = new Thread(this,this.getClass().getName());
	hierarchyThread.start();
    }

    public void run() {
	if (Thread.currentThread() == hierarchyThread) {
	    try {
		Thread.sleep(100);
	    } catch (InterruptedException e) {
	    }
	    ClassManager cm = current.classManager();
	    List list;
	    String methodName;
	    if (clsList != null) {
		list = clsList;
		methodName = null;
	    } else {
		list = mtdList;
		methodName = current.methodManager().name();
	    }
	    Enumeration en = cm.hierarchy(methodName);
	    while (en.hasMoreElements()) {
		CodePath cp = (CodePath)en.nextElement();
		codePath.addElement(cp);
		list.addItem(cp.printString());
		try {
		    Thread.sleep(100);
		} catch (InterruptedException e) {
		}
	    }
	}
    }

    /**
     * the constructor
     */
    public HierarchyBrowser(CodePath cp, CodePath select, String title) {
	super(cp, select, title);
	codePath.addElement(new CodePath(cp));
	codePath.addElement(new CodePath(select));
	String path = select.printString();
	if (clsList != null) {
	    clsList.addItem(path);
	} else {
	    mtdList.addItem(path);
	}
	selection = 0;
    }

    protected String propertyNameOfClassListWidth(String projectName) {
	return PROPERTY_LIST_WIDTH_CLASS;
    }

    public boolean handleEvent(Event ev) {
	try {
	    if ((ev.target == clsList || ev.target == mtdList) &&
		(ev.id == Event.LIST_SELECT || ev.id == Event.LIST_DESELECT)) {
		if (!acceptWithConfirm()) {
		    update();
		} else if (ev.target == mtdList && clsList != null) {
		    current.methodManager(getSelectedMethodPrototype());
		    updateCurrent();
		} else {
		    selection = ((List)ev.target).getSelectedIndex();
		    CodePath cp = (CodePath)codePath.elementAt(selection + 1);
		    if (ev.target == clsList) {
			current = cp;
			current.methodManager(getSelectedMethodPrototype());
			updateMethods();
		    } else {
			current = (selection < 0) ? startup : cp;
			updateCurrent();
		    }
		}
		return true;
	    }
	}
	catch (TeikadeException e) {
	    current.classManager((ClassManager)null);
	    TeikadeFrame.updateNotify(current);
	    update();
	    return true;
	}
	return super.handleEvent(ev);
    }

    public void updateClasses() throws TeikadeException  {
	if (clsList != null) {
	    clsList.selectAndVisible(selection);
	    clsList.reshape();
	    super.updateMethods();
        } else {
	    updateMethods();
	}
    }

    public void updateMethods() throws TeikadeException {
	if (clsList != null) {
	    super.updateMethods();
	} else {
	    mtdList.selectAndVisible(selection);
	    mtdList.reshape();
	    updateCurrent();
	}
    }

    /**
     * create menubar items
     */
    protected TeikadeMenuBar createMenuBar() {
	//debugMenu = new DebugMenu("Debug", this);
	return new HierarchyMenuBar(debugMenu);
    }
}
