/*
 * Copyright (c) 1999 PFU Limited.
 *	author SATOH Osamu
 */
package teikade.gui;

import java.awt.*;
import java.util.Hashtable;
import java.util.Enumeration;
import java.util.Properties;
import java.util.StringTokenizer;
import java.io.IOException;
import teikade.sys.SystemManager;
import teikade.sys.TeikadeProperties;
import teikade.gui.component.*;
import teikade.gui.editor.EditData;

public class MacroEditor extends DDialog {
    private static String CATEGORY_NAMES = "macro.categories";
    private static String categoryNames;

    private static Hashtable categoryMap;
    private Hashtable macroMap = null;

    static {
	categoryMap = new Hashtable();
	categoryNames = TeikadeProperties.getString(CATEGORY_NAMES);
	if (categoryNames != null) {
	    StringTokenizer names = new StringTokenizer(categoryNames, ",");
	    while (names.hasMoreTokens()) {
		String category = names.nextToken();
		String resourceName = category + ".macros";
		try {
		    Properties macroMap =
			    SystemManager.loadProperties(resourceName);
		    categoryMap.put(category, macroMap);
		} catch (IOException e) {
		    SystemManager.error("can't load macro file: " + resourceName);
		}
	    }
	}
    }

    protected EditData target;

    protected List categories          = new DList(7, false);
    protected List macros              = new DList(7, false);
    protected TextField newCategory    = new DTextField(15);
    protected TextField newMacro       = new DTextField(20);
    protected TextArea  macroContent   = new DTextArea(5, 35);
    protected Button copy              = new DButton("copy");
    protected Button paste             = new DButton("paste");
    protected Button sync              = new DButton("sync");
    //protected Button reload              = new DButton("reload");
    protected Button close             = new DButton("close");

    public MacroEditor(Frame frame, EditData target) {
	super(frame, false);
	setTitle("macro editor");
	this.target = target;

	Panel categoriesPanel = new Panel();
	categoriesPanel.setLayout(new BorderLayout());
	categoriesPanel.add("Center", categories);
	categoriesPanel.add("South", newCategory);

	Panel macrosPanel = new Panel();
	macrosPanel.setLayout(new BorderLayout());
	macrosPanel.add("Center", macros);
	macrosPanel.add("South", newMacro);

	Panel listsPanel = new Panel();
	listsPanel.setLayout(new GridLayout(1, 2));
	listsPanel.add(categoriesPanel);
	listsPanel.add(macrosPanel);

	Panel buttons = new Panel();
	buttons.add(copy);
	buttons.add(paste);
	//buttons.add(reload);
	buttons.add(sync);
	buttons.add(close);

	setLayout(new BorderLayout());
	add("North" , listsPanel);
	add("Center", macroContent);
	add("South" , buttons);

	pack();
    }

    public void show() {
	categories.clear();
	Enumeration e = categoryMap.keys();
	while (e.hasMoreElements()) {
	    String category = (String)e.nextElement();
	    categories.addItem(category);
	}

	super.show();
    }

    public boolean action(Event ev, Object arg) {
	if (ev.target == copy) {
	    copy();
	} else if (ev.target == paste) {
	    paste();
	// } else if (ev.target == reload) {
	//    reload();
	} else if (ev.target == sync) {
	    sync();
	} else if (ev.target == close) {
	    hide();
	} else if (ev.target == newCategory) {
	    newCategory();
	} else if (ev.target == newMacro) {
	    newMacro();
	} else if (ev.target == categories) {
	    categories(arg);
	} else if (ev.target == macros) {
	    macros(arg);
	} else {
	    return false;
	}
	return true;
    }

    protected void copy() {
	System.out.println("copy");
    }

    protected void paste() {
	if (!target.isEditable()) return;

	int selectionStart = target.getSelectionStart();
	int selectionEnd   = target.getSelectionEnd();
	String replaceString = macroContent.getText();
	if (selectionStart != selectionEnd) {
	    target.replaceText(replaceString, selectionStart, selectionEnd);
	} else {
	    target.insertText(replaceString, selectionStart);
	}
	target.select(selectionStart, selectionStart + replaceString.length());
    }

    protected void sync() {
	Enumeration categories = categoryMap.keys();
	while (categories.hasMoreElements()) {
	    String category = (String)categories.nextElement();
	    Properties macros = (Properties)categoryMap.get(category);
	    String resourceName = category + ".macros";
	    try {
		SystemManager.saveProperties(resourceName, macros);
	    } catch (IOException e) {
		SystemManager.error("can't save macro file: " + resourceName);
	    }
	}
    }

    protected void newCategory() {
	String categoryName = newCategory.getText();
	if (!categoryMap.containsKey(categoryName)) {
	    macroMap = new Properties();
	    categoryMap.put(categoryName, macroMap);
	    categories.addItem(categoryName);
	    categories.select(categories.countItems() - 1);
	    macros.clear();
	    categoryNames = (categoryNames == null) ?
			categoryName : categoryNames + "," + categoryName;
	    TeikadeProperties.put(CATEGORY_NAMES, categoryNames);
	} else {
	    SystemManager.beep();
	}
    }

    protected void newMacro() {
	if (macroMap == null) {
	    String categoryName = newCategory.getText();
	    if (categoryName != null) {
		newCategory();
	    } else {
		SystemManager.beep();
		return;
	    }
	}

	String macroName = newMacro.getText();
	if (!macroMap.containsKey(macroName)) {
	    macros.addItem(macroName);
	    macros.select(macros.countItems() - 1);
	}
	macroMap.put(macroName, macroContent.getText());
    }

    protected void categories(Object arg) {
	newCategory.setText((String)arg);
	macroMap = (Hashtable)categoryMap.get(arg);
	macros.clear();
	Enumeration e = macroMap.keys();
	while (e.hasMoreElements()) {
	    String macro = (String)e.nextElement();
	    macros.addItem(macro);
	}
	
    }

    protected void macros(Object arg) {
	newMacro.setText((String)arg);
	macroContent.setText((String)macroMap.get(arg));
    }
}
