/*
 * Copyright (c) 1995 PFU Limited.
 *	author Kazuhisa Shirakami
 */
package teikade.gui;

import java.awt.*;
import teikade.gui.component.*;
import teikade.gui.editor.EditData;

public class Replacer extends DDialog {
    protected EditData target;
    protected boolean forward = true;

    protected TextField searchStringField  = new DTextField(30);
    protected TextField replaceStringField = new DTextField(30);
    protected Button backwardButton    = new DButton("<< backward");
    protected Button forwardButton     = new DButton("forward >>");
    protected Button nextButton        = new DButton("next");
    protected Button replaceNextButton = new DButton("replace & next");
    protected Button replaceStayButton = new DButton("replace & stay");
    protected Button replaceAllButton  = new DButton("replace all");
    protected Button closeButton       = new DButton("done");

    public Replacer(Frame frame, EditData target) {
	super(frame, false);
	setTitle("search & replace");
	this.target = target;

	Panel labels = new Panel();
	labels.setLayout(new GridLayout(2, 1));
	labels.add(new DLabel( "search string:", Label.RIGHT));
	labels.add(new DLabel("replace string:", Label.RIGHT));
	
	Panel fields = new Panel();
	fields.setLayout(new GridLayout(2, 1));
	fields.add(searchStringField);
	fields.add(replaceStringField);
	searchStringField.setText(target.getSelectedText());

	Panel text = new Panel();
	text.setLayout(new BorderLayout());
	text.add("West", labels);
	text.add("Center", fields);

	Panel replace = new Panel();
	replace.setLayout(new GridLayout(4, 1));
	replace.add(nextButton);
	replace.add(replaceStayButton);
	replace.add(replaceNextButton);
	replace.add(replaceAllButton);

	Panel buttons = new Panel();
	buttons.setLayout(new BorderLayout());
	buttons.add("West",   backwardButton);
	buttons.add("Center", replace);
	buttons.add("East",   forwardButton);

	Panel close = new Panel();
	close.setLayout(new BorderLayout());
	close.add("Center", closeButton);

	setLayout(new BorderLayout());
	add("North" , text);
	add("Center", buttons);
	add("South" , close);

	pack();
    }

    public void setSearchString(String searchString) {
	searchStringField.setText(searchString);
    }

    public void updateActibity() {
	if (target.isEditable()) {
	    replaceStringField.setEditable(true);
	    replaceStayButton.enable();
	    replaceNextButton.enable();
	    replaceAllButton.enable();
	} else {
	    replaceStringField.setEditable(false);
	    replaceStayButton.disable();
	    replaceNextButton.disable();
	    replaceAllButton.disable();
	}
    }

    public boolean action(Event ev, Object arg) {
	if (ev.target == searchStringField ||
	    ev.target == replaceStringField) {
	    next();
	} else if (ev.target == backwardButton) {
	    backward();
	} else if (ev.target == forwardButton) {
	    forward();
	} else if (ev.target == nextButton) {
	    next();
	} else if (ev.target == replaceNextButton) {
	    replace(true);
	} else if (ev.target == replaceStayButton) {
	    replace(false);
	} else if (ev.target == replaceAllButton) {
	    replaceAll();
	} else if (ev.target == closeButton) {
	    hide();
	} else {
	    return super.action(ev, arg);
	}
	return true;
    }

    protected boolean next() {
	if (forward) {
	    return forward();
	} else {
	    return backward();
	}
    }

    protected boolean backward() {
	forward = false;

	int cursor = target.getSelectionStart();
	if (cursor <= 0) return false;

	String searchString = searchStringField.getText();
	if (searchString.length() == 0) return false;

	String code = target.getText();
	int index = code.lastIndexOf(searchString, cursor - 1);
	if (index < 0) {
	    target.select(0, 0);
	    return false;
	} else {
	    target.select(index, index + searchString.length());
	    return true;
	}
    }

    protected boolean forward() {
	forward = true;

	String searchString = searchStringField.getText();
	if (searchString.length() == 0) return false;

	String code = target.getText();
	int length = code.length();

	int cursor = target.getSelectionEnd();
	if (cursor >= length) return false;

	int index = code.indexOf(searchString, cursor);
	if (index < 0) {
	    target.select(length, length);
	    return false;
	} else {
	    target.select(index, index + searchString.length());
	    return true;
	}
    }

    protected boolean replace(boolean next) {
	if (!target.isEditable()) return false;

	int selectionStart = target.getSelectionStart();
	int selectionEnd   = target.getSelectionEnd();
	if (selectionStart != selectionEnd) {
	    String replaceString = replaceStringField.getText();
	    target.replaceText(replaceString, selectionStart, selectionEnd);
	    if (forward) selectionStart += replaceString.length();
	    target.select(selectionStart, selectionStart);
	}

	if (next) {
	    return next();
	} else {
	    return false;
	}
    }

    protected void replaceAll() {
	while (replace(true));
    }
}
