/*
 * Copyright (c) 1995 PFU Limited.
 *	author Osamu Satoh, Kazuhisa Shirakami
 */
package teikade.gui;

import teikade.sys.*;
import teikade.util.*;
import teikade.gui.component.*;
import teikade.gui.editor.EditData;
import java.awt.*;
import java.util.*;
import java.io.DataInputStream;
import java.io.IOException;
import java.io.File;

public class TeikadeGrepFrame extends ListingFrame implements GrepListener {
    private TextField patternField;
    private TextField codePathField;
    private CheckboxGroup grepCheckbox;
    Grep grep;

    public static void openOn(CodePath cp) {
        TeikadeGrepFrame grep = new TeikadeGrepFrame(cp);
	grep.open();
    }

    public static void openOn(CodePath cp, String pattern, String title) {
        TeikadeGrepFrame grep = new TeikadeGrepFrame(cp);
	grep.open();
    }

    public TeikadeGrepFrame(CodePath cp) {
	this(cp, "", "Grep");
    }

    public TeikadeGrepFrame(CodePath cp, String pattern, String title) {
	super(cp, title);
	patternField.setText(pattern);
	codePathField.setText(cp.printString());
    }

    protected Component createListingItems() {
	Panel checkPane = new Panel();
	checkPane.setLayout(new GridLayout(1, 5));
	grepCheckbox = new CheckboxGroup();
	checkPane.add(new DLabel("search engine:"));
	checkPane.add(new DCheckbox("External", grepCheckbox, false));
	checkPane.add(new DCheckbox("Simple", grepCheckbox, false));
	checkPane.add(new DCheckbox("Expression", grepCheckbox, false));
	checkPane.add(new DCheckbox("Java", grepCheckbox, true));
	codePathField = new DTextField();
	Panel codePathPane = new Panel();
	codePathPane.setLayout(new BorderLayout());
	codePathPane.add(WEST, new DLabel("search code path:"));
	codePathPane.add(CENTER, codePathField);
	patternField = new DTextField();
	Panel patternPane = new Panel();
	patternPane.setLayout(new BorderLayout());
	patternPane.add(WEST, new DLabel("egrep pattern:"));
	patternPane.add(CENTER, patternField);
	Panel grepPane = new Panel();
	grepPane.setLayout(new BorderLayout());
	grepPane.add(NORTH, codePathPane);
	grepPane.add(CENTER, checkPane);
	grepPane.add(SOUTH, patternPane);
	Panel panel = new Panel();
	panel.setLayout(new BorderLayout());
	panel.add(NORTH, grepPane);
	panel.add(CENTER, super.createListingItems());
	return panel;
    }

    public boolean action(Event ev, Object arg) {
	if (ev.target == patternField) {
	    String pattern = patternField.getText();
	    if (pattern.length() > 0) {
		start();
	    }
	    return true;
	} else {
	    boolean ret = super.action(ev, arg);
	    return ret;
	}
    }

    public void start() {
	syncAndClearListItems();
	stop();
	String cls = "teikade.util." + grepCheckbox.getCurrent().getLabel() + "Grep";
	try {
	    grep = (Grep)Class.forName(cls).newInstance();
	} catch (Exception e) {
	    addMessage(e.toString());
	    return;
	}
	grep.setCodePath(new CodePath(codePathField.getText()));
	grep.setPattern(patternField.getText());
	grep.setGrepListener(this);
	grep.execute();
    }

    public void stop() {
	if (grep != null) {
	    grep.abort();
	}
    }

    public void executing(Command grep) {
	addMessageWithDate("grep started");
    }

    public void finished(Command grep, int status) {
	if (status == 0) {
	    addMessageWithDate("grep finished");
	} else {
	    addMessageWithDate("grep finished abnormaly");
	}
	grep = null;
    }

    public void aborted(Command grep, Exception e) {
	addMessageWithDate("grep aborted: " + e.getMessage());
	grep = null;
    }

    public void result(Grep grep, CodeLocation loc) {
	addItem(loc);
    }

    public void message(Grep grep, String message) {
	addMessage(message);
    }
}
