/*
 * Copyright (c) 1995-1998 PFU Limited.
 *	author Kazuhisa Shirakami
 */
package teikade.gui.component;

import java.awt.*;
import teikade.sys.SystemManager;
import teikade.sys.TeikadeProperties;
import teikade.sys.UseProperties;
import teikade.util.ThreadUtil;

class DelegateEventDialog extends Dialog implements EventHandler, UseProperties {
    private EventDelegater eventDelegater = new EventDelegater(this);
    Thread suspendThread = null;

    DelegateEventDialog(Frame frame, boolean modal) {
	super(frame, modal);
	updateProperties();
    }

    public void updateProperties() {
	Font font = TeikadeProperties.getFont("font.dialog");
	if (font != null && !font.equals(getFont())) {
	    setFont(font);
	}
    }

    public final boolean postEvent(Event ev) {
	if (eventDelegater != null) {
	    return eventDelegater.delegate(ev);
	} else {
	    return postEvent0(ev);
	}
    }

    public boolean postEvent0(Event ev) {
	return super.postEvent(ev);
    }

    public void show() {
	super.show();
	if (SystemManager.onWin32 && isModal() && isVisible()) {
	    suspend();
	}
    }

    public void hide() {
	if (SystemManager.onWin32 && isModal()) {
	    resume();
	}
	super.hide();
    }

    private void suspend() {
	resume();
	suspendThread = Thread.currentThread();
	//suspendThread.suspend();
	ThreadUtil.Suspend(suspendThread);
    }

    private void resume() {
	if (suspendThread != null) {
	    //suspendThread.resume();
	    ThreadUtil.Resume(suspendThread);
	    suspendThread = null;
	}
    }

    public void dispose() {
	if (SystemManager.onWin32 && isModal()) {
	    resume();
	}
	eventDelegater.dispose();
	eventDelegater = null;
	super.dispose();
    }
}
