/*
 * Copyright (c) 1996-1998 PFU Limited.
 *	author Kazuhisa Shirakami
 */
package teikade.gui.component;

import java.awt.*;

public class SplitHandle extends Canvas {
    private static final int DEFAULT_GAP = 5;
    private int gap;
    private static Point grabOffset;
    private int cursor = Frame.MOVE_CURSOR;
    //private int oldCursor = Frame.DEFAULT_CURSOR;

    public void addNotify() {
	super.addNotify();
	LayoutManager layouter = getParent().getLayout();
	if (layouter instanceof SplitLayout) {
	    SplitLayout splitLayouter = (SplitLayout)layouter;
	    if (splitLayouter.getDirection() == SplitLayout.HORIZONTAL) {
		cursor = Frame.W_RESIZE_CURSOR;
	    } else {
		cursor = Frame.N_RESIZE_CURSOR;
	    }
	} else {
	    cursor = Frame.MOVE_CURSOR;
	}
    }

    public SplitHandle() {
	this(DEFAULT_GAP);
    }

    public SplitHandle(int gap) {
	this.gap = gap;
    }

    public Dimension preferredSize() {
	return minimumSize();
    }

    public Dimension minimumSize() {
	return new Dimension(gap, gap);
    }

    public synchronized boolean mouseEnter(Event evt, int x, int y) {
	setCursor(cursor);
	/*
	oldCursor = setCursor(cursor);
	// thanx for patch by Hoshi Takanori
	if (oldCursor == Frame.W_RESIZE_CURSOR
	 || oldCursor == Frame.N_RESIZE_CURSOR
	 || oldCursor == Frame.MOVE_CURSOR) {
	    oldCursor = Frame.DEFAULT_CURSOR;
	}
	*/
	return true;
    }

    public synchronized boolean mouseExit(Event evt, int x, int y) {
	setCursor(Frame.DEFAULT_CURSOR);
	/*
	int myCursor = setCursor(oldCursor);
	if (myCursor != this.cursor) {
	    setCursor(myCursor);
	}
	*/
	return true;
    }

    public boolean mouseDown(Event evt, int x, int y) {
	grabOffset = new Point(x, y);
	return true;
    }

    public boolean mouseUp(Event evt, int x, int y) {
	Container parent = getParent();
	if (parent != null && grabOffset != null) {
	    LayoutManager layouter = parent.getLayout();
	    if (layouter instanceof SplitLayout) {
		((SplitLayout)layouter).handleDone(parent, this,
		x - grabOffset.x, y - grabOffset.y, evt.shiftDown());
	    }
	    parent.validate();
	    grabOffset = null;
	}
	return true;
    }

    public boolean mouseDrag(Event evt, int x, int y) {
	Container parent = getParent();
	if (parent != null && grabOffset != null) {
	    LayoutManager layouter = parent.getLayout();
	    if (layouter instanceof SplitLayout) {
		((SplitLayout)layouter).handle(parent, this,
		x - grabOffset.x, y - grabOffset.y, evt.shiftDown());
		grabOffset.x = x;
		grabOffset.y = y;
	    }
	}
	return true;
    }

    private int setCursor(int cursor) {
	Frame frame = getFrame();
	if (frame != null) {
	    int oldCursor = frame.getCursorType();
	    frame.setCursor(cursor);
	    return oldCursor;
	}
	return Frame.DEFAULT_CURSOR;
    }

    private Frame getFrame() {
	Container parent = getParent();
	while (parent != null && !(parent instanceof Frame)) {
	    parent = parent.getParent();
	}
	return (Frame)parent;
    }
}
