/*
 * Copyright (c) 1995-1998 PFU Limited.
 *	author Kazuhisa Shirakami
 */
package teikade.gui.dialog;

import teikade.sys.*;
import teikade.util.*;
import teikade.gui.Transcript;
import teikade.gui.component.*;
import java.awt.*;
import java.util.StringTokenizer;

public class TeikadeDialog extends DDialog {
    protected static final String NORTH  = "North";
    protected static final String EAST   = "East";
    protected static final String SOUTH  = "South";
    protected static final String WEST   = "West";
    protected static final String CENTER = "Center";

    public static final String OK     = "Ok";
    public static final String CANCEL = "Cancel";

    private TextField textField;
    private Button leftButton;
    private Button middleButton;
    private Button rightButton;

    public static final int NONE   = 0;
    public static final int LEFT   = 1;
    public static final int MIDDLE = 2;
    public static final int RIGHT  = 3;
    private int choosenButton = NONE;

    public TeikadeDialog(Frame f, boolean modal) {
	super(f, modal);
    }

    public TeikadeDialog(Frame f, String msg) {
	super(f, true);
	init(msg, null, OK, null, CANCEL);
    }

    public TeikadeDialog(Frame f, String msg, String text) {
	super(f, true);
	init(msg, text, OK, null, CANCEL);
    }

    public TeikadeDialog(Frame f, String msg, String btn1, String btn2) {
	super(f, true);
	init(msg, null, btn1, null, btn2);
    }

    public TeikadeDialog(Frame f, String msg, String text,
		 String left, String middle, String right) {
	super(f, true);
	init(msg, text, left, middle, right);
    }

    protected void init(String msg, String text, String left, String middle, String right) {
	setBackground(Color.lightGray);

	// create message
	Panel message = null;
	if (msg != null && msg.length() > 0) {
	    Panel messages = new Panel();
	    messages.setLayout(new GridLayout(0, 1));
	    StringTokenizer lines = new StringTokenizer(msg.trim(), "\n\r");
	    while (lines.hasMoreTokens()) {
		messages.add(new DLabel(lines.nextToken()));
	    }
	    message = new Panel();
	    message.add(messages);
	}

	// create requesting text field
	if (text == null) {
	    textField = null;
	} else {
	    textField = new DTextField(text, Math.max(30, text.length()));
	}

	// create buttons
	Panel buttons = new Panel();
	buttons.setLayout(new BorderLayout());
	if (left != null) {
	    Panel leftArea = new Panel();
	    leftArea.add(CENTER, leftButton = new DButton(left));
	    buttons.add(WEST, leftArea);
	}
	if (middle != null) {
	    Panel middleArea = new Panel();
	    middleArea.add(CENTER, middleButton = new DButton(middle));
	    buttons.add(CENTER, middleArea);
	}
	if (right != null) {
	    Panel rightArea = new Panel();
	    rightArea.add(CENTER, rightButton = new DButton(right));
	    buttons.add(EAST, rightArea);
	}

	// layout display items
	setLayout(new BorderLayout());
	if (message   != null) add(NORTH, message);
	if (textField != null) add(CENTER, textField);
	add(SOUTH, buttons);
    }

    public int choose() {
	choosenButton = NONE;
	pack();
	show();
	return choosenButton;
    }

    public boolean confirm() {
	return choose() == LEFT;
    }

    public boolean handleEvent(Event ev) {
	if (ev.id == Event.WINDOW_DESTROY && choosenButton == NONE) {
	    if (rightButton != null) {
		actionRightButton();
		return true;
	    } else if (middleButton != null) {
		actionMiddleButton();
		return true;
	    }
	}
	return super.handleEvent(ev);
    }

    public boolean action(Event ev, Object arg) {
	if (choosenButton != NONE) return super.action(ev, arg);
	if (ev.target == leftButton) {
	    actionLeftButton();
	} else if (ev.target == middleButton) {
	    actionMiddleButton();
	} else if (ev.target == rightButton) {
	    actionRightButton();
	} else if (ev.target instanceof TextField) {
	    if (ev.shiftDown()) {
		actionMiddleButton();
	    } else {
		actionLeftButton();
	    }
	} else {
	    return super.action(ev, arg);
	}
	return true;
    }

    public void actionLeftButton() {
	choosenButton = LEFT;
	hide();
    }

    public void actionMiddleButton() {
	choosenButton = MIDDLE;
	hide();
    }

    public void actionRightButton() {
	choosenButton = RIGHT;
	hide();
    }

    public int choosenButton() {
	return choosenButton;
    }

    public String getText() {
	return textField == null ? null : textField.getText();
    }

    public static
    boolean confirm(Frame f, String msg) {
	return confirm(f, msg, OK, CANCEL);
    }

    public static
    boolean confirm(Frame f, String msg, String label) {
	return confirm(f, msg, label, CANCEL);
    }

    public static
    boolean confirm(Frame f, String msg, String label1, String label2) {
	TeikadeDialog dialog = new TeikadeDialog(f, msg, label1, label2);
	boolean accept = dialog.confirm();
	dialog.dispose();
	return accept;
    }

    public void information(String msg) {
	TeikadeDialog subDialog = new TeikadeDialog(getFrame(), false);
	subDialog.init(msg, null, null, OK, null);
	subDialog.confirm();
    }

    public static void information(Frame f, String msg) {
	information(f, msg, OK);
    }

    public static void information(Frame f, String msg, String btnLabel) {
	TeikadeDialog dialog = new TeikadeDialog(f, true);
	dialog.init(msg, null, null, btnLabel, null);
	dialog.confirm();
    }

    public static void information(Component comp, String msg) {
	information(getFrame(comp), msg);
    }

    public static
    String request(Frame f, String msg) {
	return request(f, msg, null);
    }

    public static
    String request(Frame f, String msg, String init) {
	TeikadeDialog dialog = new TeikadeDialog(f, msg, (init == null ? "" : init));
	boolean accept = dialog.confirm();
	String text = dialog.getText().trim();
	dialog.dispose();
	return (accept && text.length() > 0) ? text : null;
    }

    public static
    int choose(Frame f, String msg, String btn1, String btn2, String btn3) {
	TeikadeDialog dialog = new TeikadeDialog(f, msg, null, btn1, btn2, btn3);
	int chosen = dialog.choose();
	dialog.dispose();
	return chosen;
    }
}
