/*
 * Copyright (c) 1995 PFU Limited.
 *	author Osamu Satoh, Kazuhisa Shirakami
 */
package teikade.gui.menu;

import java.awt.*;
import teikade.sys.*;
import teikade.gui.*;
import teikade.gui.component.*;
import teikade.gui.dialog.*;

public class TranscriptMenuBar extends DMenuBar {
    DMenu fileMenu;
    DMenu projectMenu;
    DMenu removeMenu;
    DMenu renameMenu;
    DMenu frameMenu;

    protected static final String MENU_FILE    = "File";
    protected static final String MENU_PROJECT = "Project";
    protected static final String MENU_FRAME   = "Frame";

    protected static final String MENU_MEMORY_MONITOR = "memory monitor";
    protected static final String MENU_NEW_PROJECT    = "new project";
    protected static final String MENU_OPEN_CHANGES   = "open changes";
    protected static final String MENU_PROPERTIES     = "properties";
    protected static final String MENU_QUIT           = "quit";
    protected static final String MENU_REMOVE_PROJECT = "remove project";
    protected static final String MENU_RENAME_PROJECT = "rename project";
    protected static final String MENU_SPAWN_PROJECT  = "spawn ";

    public TranscriptMenuBar() {
	fileMenu    = createFileMenu();
	projectMenu = createProjectMenu();
	frameMenu  = createFrameMenu();
	add(fileMenu);
	add(projectMenu);
	add(frameMenu);
    }

    public DMenu createFileMenu() {
	DMenu menu = new DMenu(MENU_FILE);
	menu.add(MENU_OPEN_CHANGES);
	menu.add(MENU_MEMORY_MONITOR);
	menu.add(MENU_PROPERTIES);
	menu.add(MENU_QUIT);
	return menu;
    }

    public DMenu createProjectMenu() {

	removeMenu = new DMenu(MENU_REMOVE_PROJECT);
	removeMenu.disable();
	renameMenu = new DMenu(MENU_RENAME_PROJECT);
	renameMenu.disable();

	DMenu menu = new DMenu(MENU_PROJECT);
	menu.add(MENU_NEW_PROJECT);
	menu.add(removeMenu);
	menu.add(renameMenu);
	menu.addSeparator();
	// spawn project menu will be added by SystemManager call addProject()
	return menu;
    }

    public DMenu createFrameMenu() {
	return new DMenu(MENU_FRAME);
    }

    public void addProject(String projname) {
	projectMenu.add(MENU_SPAWN_PROJECT + projname);
	removeMenu.add(projname);
	removeMenu.enable();
	renameMenu.add(projname);
	renameMenu.enable();
    }

    public void renameProject(String name, String newName) {
	int numItems = projectMenu.countItems();
	for (int i = 3; i < numItems; i++) {
	    MenuItem item = projectMenu.getItem(i);
	    if (item.getLabel().endsWith(" " + name)) {
		item.setLabel(MENU_SPAWN_PROJECT + newName);
		break;
	    }
	}
	numItems = removeMenu.countItems();
	for (int i = 0; i < numItems; i++) {
	    MenuItem item = removeMenu.getItem(i);
	    if (item.getLabel().equals(name)) {
		item.setLabel(newName);
		break;
	    }
	}
	numItems = renameMenu.countItems();
	for (int i = 0; i < numItems; i++) {
	    MenuItem item = renameMenu.getItem(i);
	    if (item.getLabel().equals(name)) {
		item.setLabel(newName);
		break;
	    }
	}
    }

    public void addFrame(Frame frame) {
	frameMenu.add(new FrameRaiseMenuItem(frame));
    }

    public void removeFrameAt(int index) {
	frameMenu.remove(index + 1);	// +1 for Transcript
    }

    public boolean action(Event ev, Object arg, Transcript transcript) {
	if (ev.target instanceof MenuItem) {
	    Menu menu = (Menu)((MenuItem)ev.target).getParent();
	    String label = (String)arg;
	    if (menu == fileMenu) {
		if (label.equals(MENU_OPEN_CHANGES)) {
		    TeikadeFile changesFile = SystemManager.changesFile();
		    String dirname = changesFile.getParent();
		    String fname = changesFile.getName();
		    fname = SystemManager.requestFile("Changes file to open", dirname, fname);
		    if (fname != null) {
			ChangeListFrame.openOn(fname);
		    }
		} else if (label.equals(MENU_MEMORY_MONITOR)) {
		    transcript.memoryMonitor();
		} else if (label.equals(MENU_PROPERTIES)) {
		    transcript.properties();
		} else if (label.equals(MENU_QUIT)) {
		    transcript.close();
		}
		return true;
	    } else if (menu == projectMenu) {
		if (label.equals(MENU_NEW_PROJECT)) {
		    NewProjectDialog newProject = new NewProjectDialog(transcript);
		    if (!newProject.confirm()) return true;
		    ProjectManager pm =
			SystemManager.createProject(newProject.projectName(),
					     newProject.projectPath(),
					     newProject.rebuildNow());
		    if (pm != null && newProject.spawnNow()) {
			Browser.openOn(pm.getCodePath());
		    }
		} else if (label.startsWith(MENU_SPAWN_PROJECT)) {
		    String proj = label.substring(MENU_SPAWN_PROJECT.length());
		    CodePath cp = new CodePath(null,
					    SystemManager.findProject(proj),
					    null, null, null);
		    Browser.openOn(cp);
		}
		return true;
/*
	    } else if (menu == frameMenu) {
		int colon = label.indexOf(':');
		int index = Integer.parseInt(label.substring(0, colon));
		if (index == 0) {
		    Transcript.raise();
		} else {
		    TeikadeFrame.frameAt(index - 1).show();
		}
		return true;
 */
	    } else if (menu == removeMenu) {
		if (transcript.removeProject(label)) {
		    removeMenu.remove((MenuComponent)ev.target);
		    renameMenu.remove(renameMenu.indexOf((String)arg));
		    projectMenu.remove(projectMenu.indexOf("spawn " + (String)arg));
/*
		    while (--n >= 0) {
			if (label.equals(removeMenu.getItem(n).getLabel())) {
			    projectMenu.remove(projectMenu.indexOf("spawn " + );
			    removeMenu.remove(n);
			    break;
			}
		    }
*/
		    if (removeMenu.countItems() == 0) removeMenu.disable();
		}
		return true;
	    } else if (menu == renameMenu) try {
		SystemManager.findProject(label).rename();
		return true;
	    }
	    catch (TeikadeException e) {
	    }
	}
	return false;
    }
}
