/*
 * Copyright (c) 1996 PFU Limited.
 *	author Kazuhisa Shirakami
 */
package teikade.lang;

import java.lang.*;
import java.io.*;
import java.util.*;

public class BackwardExpression {
    private String code;
    private int codeLength;
    private int codePointer;

    private int startPoint = 0;	 	// start pointer
    private int endPoint   = 0;		// next start pointer
    private int keyPoint   = 0;		// keyword pointer

    public BackwardExpression(String str, int start) {
	code = str;
	codeLength = str.length();
	codePointer = start;
    }

    public boolean nextExpression() throws DescriptionException {
	if (codePointer <= 0) return false;
	startPoint = codePointer;
	while (codePointer > 0 && onSpace()) codePointer--;
	keyPoint = codePointer;
	if (onBeginningOfPair()) {
	    codePointer = 0;
	    throw new DescriptionException("unmatched balance");
	}
	if (onEndOfPair()) {
	    nextPair();
	    endPoint = codePointer;
	    return true;
	}
	while (!onWord() && !onSpace() && !onBeginningOfPair() && !onEndOfPair()) {
	    codePointer--;
	}
	while (onWord()) codePointer--;
	endPoint = codePointer;
	return true;
    }

    boolean onSpace() {
	return code.charAt(codePointer - 1) <= ' ';
    }

    boolean onWord() {
	return Character.isJavaLetterOrDigit(code.charAt(codePointer - 1));
    }

    boolean onBeginningOfPair() {
	switch (code.charAt(codePointer - 1)) {
	case '(':
	case '{':
	case '[':
	    return true;
	case '/':
	case '*':
	    if (codePointer < 2
	    ||  code.charAt(codePointer - 2) != '/') return false;
	    return true;
	}
	return false;
    }

    boolean onEndOfPair() {
	switch (code.charAt(codePointer - 1)) {
	case ')':
	case '}':
	case ']':
	case '"':
	case '\'':
	    return true;
	case '/':
	    if (codePointer < 2
	    ||  code.charAt(codePointer - 2) != '*') return false;
	    return true;
	}
	return false;
    }

    public boolean nextPair() throws DescriptionException {
	startPoint = codePointer;
	while (codePointer > 0 && !onEndOfPair()) {
	    if (onBeginningOfPair()) {
		codePointer = 0;
		throw new DescriptionException("unmatched balance");
	    }
	    codePointer--;
	}
	keyPoint = codePointer;
	if (codePointer <= 0) {
	    endPoint = codePointer;
	    codePointer = 0;
	    return false;
	}
	skipPair();
	endPoint = codePointer;
	return true;
    }

    boolean skipPair() throws DescriptionException {
	switch (code.charAt(codePointer - 1)) {
	case ')': codePointer--; skipPair('('); return true;
	case '}': codePointer--; skipPair('{'); return true;
	case ']': codePointer--; skipPair('['); return true;
	case '"': codePointer--; skipQuote('"'); return true;
	case '\'': codePointer--; skipQuote('\''); return true;
	case '/':
	    codePointer = code.lastIndexOf("/*", codePointer - 2);
	    if (codePointer < 0) {
		codePointer = 0;
		throw new DescriptionException("not found comment end (\"*/\")");
	    }
	    endPoint = codePointer;
	    return true;
	}
	return false;
    }

    public boolean skipPair(char pair) throws DescriptionException {
	while (codePointer > 0) {
	    if (code.charAt(codePointer - 1) == pair) {
		codePointer--;
		return true;
	    }
	    if (onBeginningOfPair()) break;
	    if (onEndOfPair()) {
		nextPair();
	    } else {
		codePointer--;
	    }
	}
	codePointer = 0;
	throw new DescriptionException("unmatched balance");
    }

    public boolean skipQuote(char pair) throws DescriptionException {
	while (codePointer > 0) {
	    codePointer--;
	    if (codePointer > 0 && code.charAt(codePointer - 1) == '\\') {
		codePointer--;
	    } else if (code.charAt(codePointer) == pair) {
		return true;
	    }
	}
	codePointer = 0;
	throw new DescriptionException("unmatched balance");
    }

    public int startPoint() {
	return startPoint;
    }

    public int endPoint() {
	return endPoint;
    }

    public int keyPoint() {
	return keyPoint;
    }
}
