/*
 * Copyright (c) 1996 PFU Limited.
 *	author Kazuhisa Shirakami
 */
package teikade.lang;

import java.lang.*;
import java.io.*;
import java.util.*;

public class ForwardExpression {
    private String code;
    private int codeLength;
    private int codePointer;

    private int startPoint = 0;	 	// start pointer
    private int endPoint   = 0;		// next start pointer
    private int keyPoint   = 0;		// keyword pointer

    public ForwardExpression(String str, int start) {
	code = str;
	codeLength = str.length();
	codePointer = start;
    }

    public boolean nextExpression() throws DescriptionException {
	if (codePointer >= codeLength) return false;
	startPoint = codePointer;
	while (codePointer < codeLength && onSpace()) codePointer++;
	keyPoint = codePointer;
	if (onEndOfPair()) {
	    throw new DescriptionException("unmatched balance");
	}
	if (onBeginningOfPair()) {
	    nextPair();
	} else {
	    while (!onWord() && !onSpace() && !onBeginningOfPair() && !onEndOfPair()) {
		codePointer++;
	    }
	    while (onWord()) codePointer++;
	}
	endPoint = codePointer;
	return true;
    }

    boolean onSpace() {
	return code.charAt(codePointer) <= ' ';
    }

    boolean onWord() {
	return Character.isJavaLetterOrDigit(code.charAt(codePointer));
    }

    boolean onBeginningOfPair() {
	switch (code.charAt(codePointer)) {
	case '(':
	case '{':
	case '[':
	case '"':
	case '\'':
	    return true;
	case '/':
	    if (codePointer + 1 == codeLength) return false;
	    switch (code.charAt(codePointer + 1)) {
	    case '/':
	    case '*':
		return true;
	    }
	}
	return false;
    }

    boolean onEndOfPair() {
	switch (code.charAt(codePointer)) {
	case ')':
	case '}':
	case ']':
	    return true;
	case '*':
	    if (codePointer + 1 == codeLength) return false;
	    if (code.charAt(codePointer + 1) == '/') return true;
	}
	return false;
    }

    public boolean nextPair() throws DescriptionException {
	startPoint = codePointer;
	while (codePointer < codeLength && !onBeginningOfPair()) {
	    if (onEndOfPair()) {
		throw new DescriptionException("unmatched balance");
	    }
	    codePointer++;
	}
	keyPoint = codePointer;
	if (codePointer >= codeLength) {
	    endPoint = codePointer;
	    code = null;
	    codeLength = 0;
	    return false;
	}
	skipPair();
	endPoint = codePointer;
	return true;
    }

    boolean skipPair() throws DescriptionException {
	switch (code.charAt(codePointer)) {
	case '(': codePointer++; skipPair(')'); return true;
	case '{': codePointer++; skipPair('}'); return true;
	case '[': codePointer++; skipPair(']'); return true;
	case '"': codePointer++; skipQuote('"'); return true;
	case '\'': codePointer++; skipQuote('\''); return true;
	case '/':
	    codePointer++;
	    switch (code.charAt(codePointer++)) {
	    case '/':
		char c;
		while (codePointer < codeLength) {
		    c = code.charAt(codePointer);
		    if (c == '\n' || c == '\r') {
			return true;
		    }
		    codePointer++;
		}
		code = null;
		return true;
	    case '*':
		while (codePointer < codeLength
		&&     (codePointer = code.indexOf('*', codePointer) + 2) > 0) {
		    if (code.charAt(codePointer - 1) == '/') {
			endPoint = codePointer;
			return true;
		    }
		    codePointer++;
		}
		code = null;
		codeLength = 0;
		throw new DescriptionException("not found comment end (\"*/\")");
	    }
	    return false;
	}
	return false;
    }

    public boolean skipPair(char pair) throws DescriptionException {
	while (codePointer < codeLength) {
	    if (code.charAt(codePointer) == pair) {
		codePointer++;
		return true;
	    }
	    if (onEndOfPair()) break;
	    if (onBeginningOfPair()) {
		nextPair();
	    } else {
		codePointer++;
	    }
	}
	code = null;
	codeLength = 0;
	throw new DescriptionException("unmatched balance");
    }

    public boolean skipQuote(char pair) throws DescriptionException {
	while (codePointer < codeLength) {
	    char c = code.charAt(codePointer);
	    if (c == pair) {
		codePointer++;
		return true;
	    }
	    if (c == '\\') {
		codePointer++;
		if (codePointer == codeLength) break;
	    }
	    codePointer++;
	}
	code = null;
	codeLength = 0;
	throw new DescriptionException("unmatched balance");
    }

    public int startPoint() {
	return startPoint;
    }

    public int endPoint() {
	return endPoint;
    }

    public int keyPoint() {
	return keyPoint;
    }
}
