/**
 * Sat, 13 Nov 1999 15:39:51 +0100
 *	This program is offered by Maurizio Scaglione.
 *
 * If anyone would like to make use of the NewPluginCommand.class (see
 * attachment) just copy into the package teikade.plugin and add the line
 * 'plugin.create=teikade.plugin.NewPluginCommand' to the
 * $HOME/.teikade/.properties file. You can then invoke it and
 * create/install whatever plugin you desire for your system.
 *	
 */
package teikade.plugin;

import teikade.sys.*;
import teikade.lang.AccessFlags;
import teikade.lang.ClassSpec;
import teikade.lang.JavaLexical;
import teikade.gui.component.*;
import teikade.gui.Browser;
import teikade.gui.TeikadeFrame;
import teikade.gui.dialog.TeikadeDialog;
import java.awt.*;

public class NewPluginCommand implements PluginCommand {

    /**
     * teikade.sys.PluginCommand stuff
     */
    public void initialize(String name) throws PluginException  {
    }

    /**
     * teikade.sys.PluginCommand stuff
     */
    public void invoke(String name, Object invoker, CodePath target) throws PluginException {
	Browser frame = null;
	try {
	    frame = (Browser)invoker;
	} catch (ClassCastException e) {
	    SystemManager.errorln("Error: cannot create plugin.");
	    return;
	}
	
	if (!frame.acceptWithConfirm()) return;
	NewPluginDialog dialog = (new NewPluginDialog(frame, target.packageCodePath(), new ClassSpec(null, null, null, false)));
	if (dialog.confirm() && frame.acceptWithConfirm()) {
	    ClassSpec cs = dialog.getClassSpec();
	    ClassManager cm = target.packageManager().createClass(cs);
	    if (cm != null) {
                cm.sync();
		TeikadeFrame.updateNotify(cm.getCodePath());
		if (TeikadeDialog.confirm(frame, "Do you want to install the plugin?", "Install")) {
                    String plugin = TeikadeDialog.request(frame, "Plugin name:");
                    if (plugin != null && plugin.length() > 0) {
                        plugin = "plugin." + plugin;
                        TeikadeProperties.put(plugin, cm.fullClassName());
			TeikadeDialog.information(frame, "Installation successfull.\nPlease compile the class \'"
                                             + cm.fullClassName() + "\' and restart teikade.");
		    }
		}
	    }
	}
	dialog.dispose();
    }

    /**
     * teikade.sys.PluginCommand stuff
     */
    public boolean isInvocableFor(String name, Object invoker, CodePath target)  {
	return target.packageManager() != null;
    }
}

class NewPluginDialog extends TeikadeDialog {
    CodePath	codePath;
    ClassSpec	classSpec;
    TextField	className;
    TextField	superClassName;
    TextField	interfaceNames;
    Checkbox	isPublic;
    Checkbox	isFinal;
    Checkbox	isAbstract;

    public NewPluginDialog(Frame f, CodePath cp, ClassSpec cs) {
	super(f, "new " + cs.typeAsString());

	codePath = cp;
	classSpec = cs;

	int rows =  3;

	Panel labels = new Panel();
	labels.setLayout(new GridLayout(rows, 1));
	labels.add(new DLabel("new plugin", Label.RIGHT));
	labels.add(new DLabel(ClassSpec.STR_EXTENDS, Label.RIGHT));
	if (rows == 3) {	// is class
	    labels.add(new DLabel(ClassSpec.STR_IMPLEMENTS, Label.RIGHT));
	}

	Panel fields = new Panel();
	fields.setLayout(new GridLayout(rows, 1));
	fields.add(className = new DTextField(cs.className(), 30));
	fields.add(superClassName = new DTextField(cs.superClassName(), 30));
	if (rows == 3) {	// is class
	    fields.add(interfaceNames = new DTextField(cs.interfaceNames(), 30));
	}

	Panel modifiers = new Panel();
	modifiers.setLayout(new FlowLayout(FlowLayout.LEFT));
	isFinal = new DCheckbox(AccessFlags.STR_FINAL, null, cs.isFinal());
	modifiers.add(isFinal);
	
	Panel spec = new Panel();
	spec.setLayout(new BorderLayout());
	spec.add(WEST, labels);
	spec.add(CENTER, fields);

	Panel panel = new Panel();
	panel.setLayout(new BorderLayout());
	panel.add(NORTH, modifiers);
	panel.add(CENTER, spec);

	add(CENTER, panel);
	pack();
    }

    public ClassSpec getClassSpec() {
	return classSpec;
    }

    public void actionLeftButton() {
	String cn = className.getText().trim();
	if (cn.length() == 0) return;
	int illegal = JavaLexical.isIdentifier(cn);
	if (illegal >= 0) {
	    information("illegal character '" + cn.charAt(illegal) + "' for " +
					   classSpec.typeAsString() + " name");
	    return;
	}
	String scn = superClassName.getText().trim();
	if (scn.length() == 0) scn = null;
	String in = interfaceNames.getText().trim();
	in += (in.length() > 0) ? ", PluginCommand" : "PluginCommand";
	classSpec = new ClassSpec(cn, scn, in, false);
	classSpec.setPublic(true);
	classSpec.setFinal(isFinal.getState());
	super.actionLeftButton();
    }
}
