/*
 * Copyright (c) 1995-1998 PFU Limited.
 *	author Osamu Satoh
 */
package teikade.sys;

import java.io.File;

/**
 * abstract class for path based managers
 */
abstract public class PathedManager extends Manager {
    /**
     * the relative path for this Manager
     */
    String path;

    /**
     * the file object
     */
    TeikadeFile file;

    /**
     * when the file is modified
     */
    //long lastModified = -1;

    protected PathedManager(Manager parent, String name, String path) {
	super(parent, name);
	this.path = path;
	this.file = null;
    }

    /**
     * set new path
     */
    protected void setPath(String newPath) {
	path = newPath;
	file = null;
    }

    /**
     * get relative pathname
     */
    public String path() {
	return path;
    }

    /**
     * get absolute pathname
     */
    public String absolutePath() {
	if (parent != null && parent instanceof PathedManager) {
	    return ((PathedManager)parent).absolutePath() + File.separator + path;
	} else {
	    return path;
	}
    }

    /**
     * get File object
     */
    public TeikadeFile file() {
	if (file == null) {
	    file = TeikadeFile.create(absolutePath());
	}
	return file;
    }

    /**
     * get File object
     */
    public TeikadeFile directory() {
	if (file().isDirectory()) {
	    return file();
	} else {
	    return TeikadeFile.create(file().getParent());
	}
    }

    /**
     * dispose me
     */
    protected void dispose() {
	if (path != null) {
	    path = null;
	}
	if (file != null) {
	    //file = null; // will be used on sync()
	}
	super.dispose();
    }
}
