/*
 * Copyright (c) 1998 PFU Limited.
 *      author Kazuhisa Shirakami
 */
package teikade.util;

import teikade.sys.CodePath;
import teikade.sys.SystemManager;
import teikade.sys.PackageManager;
import teikade.sys.SourceFileManager;
import teikade.sys.TeikadeProperties;
import java.util.Vector;
import java.util.Enumeration;
import java.util.StringTokenizer;
import java.io.File;

public abstract class CompileExternal extends Compile {
    protected static final String COMMAND = "compile.command";
    protected static final String OPTIONS = "compile.options";
    protected static final String CLASSPATH = "compile.class.path";
    protected static final String SYSTEM_CLASSPATH = "java.class.path";

    CommandExternal commandExternal;

    protected CompileTarget createCompileTarget() {
	return new CompileExternalTarget(this);
    }

    public synchronized void execute() {
	if (commandExternal != null) {
	    throw new IllegalThreadStateException("already executing");
	}
	commandExternal = makeCommandExternal();
	commandExternal.execute();
    }

    public synchronized void abort() {
	if (commandExternal == null) {
	    throw new IllegalThreadStateException("already stopped");
	}
	commandExternal.abort();
	commandExternal = null;
    }

    protected CommandExternal makeCommandExternal() {
	return new CommandExternal(makeCommandExternalCommand(),
				   makeCommandExternalListener());
    }

    public String[] makeCommandExternalCommand() {
	Vector command = new Vector();
	addCommandName(command);
	addCommandOptions(command);
	addListItems(command);
	String[] commandArray = new String[command.size()];
	command.copyInto(commandArray);
	return commandArray;
    }

    protected void addCommandName(Vector command) {
	command.addElement(getCommandName());
    }

    protected void addCommandOptions(Vector command) {
	String options = getCommandOptions();
	if (options == null) {
	    return;
	}
	StringTokenizer tokens = new StringTokenizer(options);
	while (tokens.hasMoreTokens()) {
	    command.addElement(tokens.nextToken());
	}
    }

    protected void addListItems(Vector command) {
	Enumeration files = getCompileTarget().getItems();
	while (files.hasMoreElements()) {
	    command.addElement(files.nextElement());
	}
    }

    protected abstract CommandExternalListener makeCommandExternalListener();

    public String getPropertyName(String name) {
	String projectName = getProjectName();
	if (projectName == null) {
	    return name + ".default";
	} else if (CompileFactory.USER_DEFAULT.equals(projectName)) {
	    return name;
	} else {
	    return name + '.' + projectName;
	}
    }

    protected String getClassPathPropertyName() {
	if (isSystemDefault()) {
	    return SYSTEM_CLASSPATH;
	} else {
	    return getPropertyName(CLASSPATH);
	}
    }

    protected String getCommandPropertyName() {
	return getPropertyName(COMMAND);
    }

    protected String getOptionsPropertyName() {
	return getPropertyName(OPTIONS);
    }

    public CompileProperties getCompileProperties() {
	CompileProperties properties = new CompileProperties();
	String compilePropertyName = CompileFactory.getCompilePropertyName(getProjectName());
	if (compilePropertyName != null) {
	    properties.add(compilePropertyName,
			   "compile class",
			   getClass().getName());
	}
	properties.add(getCommandPropertyName(),
		       "command name",
		       getDefaultCommandName());
	properties.add(getOptionsPropertyName(),
		       "command options",
		       getDefaultCommandOptions());
	properties.add(getClassPathPropertyName(),
		       '%' + getClassPathPropertyName() + '%',
		       getDefaultClassPath());
	if (compilePropertyName == null) {
	    properties.setEditable(false);
	}
	return properties;
    }

    protected String getDefaultClassPath() {
	if (isSystemDefault()) {
	    return TeikadeProperties.getString(SYSTEM_CLASSPATH);
	} else if (isUserDefault()) {
	    String classpath = TeikadeProperties.getParsedString(CLASSPATH);
	    return classpath != null ? classpath : "";
	} else {
	    return getCompileTarget().getClassPath();
	}
    }

    protected abstract String getDefaultCommandName();

    protected abstract String getDefaultCommandOptions();

    public String getCommandName() {
	return getCompileProperties().getParsedString(getCommandPropertyName());
    }

    public String getCommandOptions() {
	return getCompileProperties().getParsedString(getOptionsPropertyName());
    }
}
