/*
 * Copyright (c) 1998 PFU Limited.
 *      author Kazuhisa Shirakami
 */
package teikade.util;

import teikade.sys.CodePath;
import teikade.sys.SystemManager;
import teikade.sys.PackageManager;
import teikade.sys.SourceFileManager;
import teikade.sys.TeikadeProperties;
import java.util.Vector;
import java.util.Enumeration;
import java.io.File;

public class CompileExternalTarget implements CompileTarget {
    private Compile compile;
    private final Vector items = new Vector();
    private String classPath;

    public CompileExternalTarget(Compile compile) {
	this.compile = compile;
    }

    public Compile getCompile() {
	return compile;
    }

    public CodePath getCodePath() {
	return getCompile().getCodePath();
    }

    public boolean isUpdatedItems() {
	return getCompile().isUpdatedTarget();
    }

    public void update() {
        clearItems();
	if (getCompile().isSystemDefault()
	 || getCompile().isUserDefault()) {
	    return;
	}
	CodePath codePath = getCodePath();
        if (codePath.sourceFileManager() != null) {
            classPath = codePath.sourceFileManager().getClassPath();
            String file = codePath.sourceFileManager().absolutePath();
            addItem(file);
        } else if (codePath.packageManager() != null) {
            classPath = codePath.projectManager().absolutePath();
            PackageManager pm = codePath.packageManager();
            Enumeration en = pm.sources();
            while (en.hasMoreElements()) {
		SourceFileManager sfm = (SourceFileManager)en.nextElement();
		if (!isUpdatedItems() || sfm.isUpdatedAfterLastCompilation()) {
		    addItem(sfm.absolutePath());
		}
            }
        } else {
            classPath = codePath.projectManager().absolutePath();
	}
	setClassPath(classPath);
	limitItemsLength(getItemsLimitLength());
    }

    protected void clearItems() {
	items.setSize(0);
    }

    protected void addItem(String name) {
	items.addElement(name);
    }

    public Enumeration getItems() {
	return items.elements();
    }

    public int getItemsCount() {
	return items.size();
    }

    public String getItemAt(int index) {
	return (String)items.elementAt(index);
    }

    protected void setClassPath(String path) {
	classPath = path;
    }

    public String getClassPath() {
	return classPath;
    }

    protected void limitItemsLength(int limitLength) {
	if (limitLength > 0 && getItemsLength() > limitLength) {
	    clearItems();
	    addItem(getCodePath().packageManager().absolutePath() + File.separator + "*.java");
	    getCompile().result(CompileResult.createMessage(
		getCompile().isUpdatedTarget()
		    ? "Sorry, compile all sources using a wildcard character.\n"
		    : "Sorry, use a wildcard character for compile\n"));
	}
    }

    protected int getItemsLimitLength() {
	if (!SystemManager.onWin32
	 || CommandExternal.useTeikadeShell
	 || getItemsCount() <= 1) {
	    return 0;
	}
	String prop = TeikadeProperties.getString("command.limit.length");
	if (prop != null) {
	    try {
		return Integer.parseInt(prop);
	    } catch (Exception e) {
	    }
	}
	return 255;
    }

    protected int getItemsLength() {
	int length = 0;
	int num = getItemsCount();
	for (int i = 0; i < num; i++) {
	    length += getItemAt(i).length() + 1; // 1 for a space character
	}
	return length;
    }
}
