/*
 * Copyright (c) 1998 SCK Files, PFU Limited.
 * author Maurizio Scaglione, SHIRAKAMI Kazuhisa.
 *
 * This is contributed by Maurizio Scaglione.
 * Thank you very much.
 */
package teikade.util;

import teikade.sys.*;
import java.util.*;
import java.io.InputStream;
import java.io.IOException;

public class GrepInternalSimple extends Grep {
    private Vector files;

    public synchronized void run() {
	byte[] text;
        byte[] pattern = CodeConverter.toBytes(getPattern());
        Enumeration enum = files.elements();
        while (enum.hasMoreElements()) {
            SourceFileManager src = (SourceFileManager)enum.nextElement();
            try {
		TeikadeFile file = TeikadeFile.create(src.absolutePath());
		InputStream is = file.getInputStream();
		text = new byte[is.available()];
		is.read(text);
		is.close();
		grep(pattern, text, src);
            } catch (IOException e) {
		System.err.println(e.toString());
            }
	}
	finished(0);
    }

    private void grep(byte[] pattern, byte[] text, SourceFileManager src) {
	long patternHash = 0; // hash code for pattern
	for (int i = 0; i < pattern.length; i++) {
	    patternHash = (patternHash << 1) + pattern[i];
	}
	int fromIndex = 0, line = 1;
	for (int toIndex = 0; toIndex < text.length; toIndex++) {
	    byte c = text[toIndex];
	    if (c == '\r') {		// CR
		if (toIndex + 1 < text.length && text[toIndex + 1] == '\n') {
		    toIndex++;		// CR+LF
		}
	    } else if (c != '\n') {	// LF
		continue;
	    }
	    if (fromIndex + pattern.length < toIndex) {
		int bol = fromIndex;
		while ((fromIndex = indexOf(text, fromIndex, toIndex,
					    pattern, patternHash)) != -1) {
		    try {
			 String detail = src.path() + ":" + line + ":"
			     + CodeConverter.toString(text, bol, toIndex - bol)
					    .trim();
			 CodeLocation result = new CodeLocation(
			     src.absolutePath(), line, detail,
			     fromIndex - bol, pattern.length);
			 addResult(result);
		    } catch (CodeLocationException e) {
			 addMessage("fail toIndex search location "
			 + src.absolutePath() + ":" + line + ", sorry.\n");
		    }
		    fromIndex++;
		}
	    }
	    line++;
	    fromIndex = toIndex + 1;
	}
    }

    public void setCodePath(CodePath cp) {
	super.setCodePath(cp);
	files = getSources(cp);
    }

    private static int indexOf(byte[] text, int fromIndex, int toIndex,
			       byte[] pattern, long patternHash) {
	if (fromIndex < 0) fromIndex = 0;
	int maxIndex = toIndex - pattern.length;
	if (fromIndex > maxIndex) return -1;
	long textHash = 0;
	for (int i = 0; i < pattern.length; i++) {
	    textHash = (textHash << 1) + text[fromIndex + i];
	}
	for (int i = fromIndex; i <= maxIndex; i++) {
	    if (patternHash == textHash) { // check text
		int j = 0;
		while (j < pattern.length && pattern[j] == text[i + j]) j++;
		if (j == pattern.length) return i;
	    }
	    if (i < maxIndex) {
		textHash -= text[i] << (pattern.length - 1);
		textHash = (textHash << 1) + text[i + pattern.length];
	    }
	}
	return -1;
    }

    public String getPatternLabel() {
	return "search string:";
    }
}
