/*
 * Copyright (c) 1999 PFU Limited.
 *      author Kazuhisa Shirakami
 */
package teikade.util;

import teikade.sys.TeikadeProperties;
import teikade.sys.SystemManager;
import java.util.Vector;
import java.util.Hashtable;
import java.util.Enumeration;
import java.util.StringTokenizer;
import java.util.Properties;

public class SourceCompiler {
    private String projectName;
    private final Vector names = new Vector();
    private final Hashtable labels = new Hashtable();

    protected SourceCompiler(String projectName) {
	this.projectName = projectName;
    }

    public String getProjectName() {
	return projectName;
    }

    public void addPropertyName(String name, String label) {
	names.addElement(name);
	labels.put(name, label);
    }

    public Enumeration getPropertyNames() {
	return names.elements();
    }

    public String getPropertyLabel(String name) {
	return (String)labels.get(name);
    }

    public String getProperty(String name) {
	String property = getUserProperty(name + '.' + projectName);
	if (projectName == null) {
	    property = getUserProperty(name);
	    if (projectName == null) {
		property = getDefaultProperty(name);
	    }
	}
        return property;
    }

    public void setProperty(String name, String value) {
	if (value == null) {
	    throw new IllegalArgumentException("value is null");
	}
	if (projectName != null) {
	    name += '.' + projectName;
	}
	if (!value.equals(getProperty(name))) {
	    setUserProperty(name, value);
	}
    }

    public static String getUserProperty(String name) {
	return TeikadeProperties.getParsedString(name);
    }

    public static void setUserProperty(String name, String value) {
	TeikadeProperties.putWithoutNotify(name, value);
    }

    public static void setDefaultProperty(String name, String value) {
	TeikadeProperties.setDefault(name, value);
    }

    public static String getDefaultProperty(String name) {
	return TeikadeProperties.getDefault(name);
    }
}
