package teikade.util;

import java.awt.*;
import java.io.*;
import java.util.*;
import teikade.sys.TeikadeFile;
import teikade.gui.component.DFrame;
import teikade.gui.component.DMenu;
import teikade.gui.component.DMenuBar;
import teikade.gui.component.DTextArea;
import teikade.gui.component.DTextField;
import teikade.util.CodeConverter;

public class Terminal extends DFrame implements Runnable {
    /* GUI items */
    MenuBar menuBar;
    Menu fileMenu, editMenu, optionMenu;
    TextField textField;
    TextArea textArea;

    /* items for subprocess */
    Thread readThread, errorThread;
    Process process;
    DataInputStream in, err;
    PrintStream out;
    byte buf[] = new byte[1024];
    Hashtable environments = new Hashtable();
    boolean echoOn = true;

    /* others */
    Vector history = new Vector();
    String command;
    String writeFile = "";
    String readFile = "";
    String directory = System.getProperty("user.dir");

    public Terminal() {
	super("Input/Output Terminal");
	setLayout(new BorderLayout());
	setMenuBar(menuBar = new DMenuBar());

	fileMenu = menuBar.add(new DMenu("File"));
	fileMenu.add("read from");
	fileMenu.add("write on");
	fileMenu.add("toggle echo");
	fileMenu.add("start");
	fileMenu.add("stop");
	fileMenu.add("close");

	textArea = new DTextArea(15, 50);
	textField = new DTextField(50);
	setLayout(new BorderLayout());
	add("Center", textArea);
	add("South", textField);
	textArea.setEditable(false);

	pack();
    }

    public Terminal(String command) throws IOException {
	this();
	this.start(command);
    }

    public Terminal(String command, String title) throws IOException {
	this(command);
	this.setTitle(title);
    }

    /**
     * java.lang.Runnable stuff
     */
    public void run() {
	Thread currentThread = Thread.currentThread();
	if (currentThread == readThread) {
	    try {
		int len;
		/*
		while (process != null && (len = in.read(buf)) != -1) {
		    textArea.appendText(CodeConverter.toString(buf, 0, len));
		}
		*/
		while ((len = in.read(buf)) != -1) {
		    textArea.appendText(CodeConverter.toString(buf, 0, len));
		}
	    } catch (IOException e) {
	    } finally {
		readThread = null;
		if (process != null) stop();
	    }
	} else if (currentThread == errorThread) {
	    try {
		int len;
		while (process != null && (len = err.read(buf)) != -1) {
		    textArea.appendText(CodeConverter.toString(buf, 0, len));
		}
	    } catch (IOException e) {
	    } finally {
		errorThread = null;
	    }
	}
    }

    /**
     * Called if an action occurs in the Component
     * @param evt the event
     * @param what the action that's occuring
     * @see java.awt.Component#action
     */
    public boolean action(Event evt, Object what) {
	if (evt.target == textField) {
	    String text = textField.getText();
	    history.addElement(text);
	    if (shouldEcho()) textArea.appendText("> " + text + "\n");
	    textField.setText("");
	    if (process != null) {
		out.println(text);
		out.flush();
	    }
	    return true;
	} if (evt.target instanceof MenuItem) {
	    return handleMenu(((MenuItem)evt.target).getParent(), (String)what);
	}
	return super.action(evt, what);
    }

    private boolean handleMenu(MenuContainer menu, String label) {
	if (menu == fileMenu) {
	    if (label.equals("read from")) {
		readFrom();
	    } else if (label.equals("write on")) {
		writeOn();
	    } else if (label.equals("toggle echo")) {
		echoOn = !echoOn;
	    } else if (label.equals("start")) {
		try {
		    start();
		} catch (IOException e) {
		}
	    } else if (label.equals("stop")) {
		stop();
	    } else if (label.equals("close")) {
		stop();
		hide();
	    } else {
		return false;
	    }
	    return false;
	}
	return false;
    }

    private void readFrom() {
	FileDialog dialog = new FileDialog(this, "Read Input from:");
	dialog.setDirectory(directory);
	dialog.setFile(readFile);
	dialog.show();
	TeikadeFile file = TeikadeFile.create(dialog.getDirectory() + File.separator + dialog.getFile());
	try {
	    InputStream is = file.getInputStream();
	    try {
		byte buf[] = new byte[is.available()];
		is.read(buf);
		out.write(buf);
		out.flush();
	    } finally {
		is.close();
	    }
	} catch (IOException e) {
	    // we should do something, but what?
	}
    }

    private void writeOn() {
	FileDialog dialog = new FileDialog(this, "Write Log on:", FileDialog.SAVE);
	dialog.setDirectory(directory);
	dialog.setFile(writeFile);
	dialog.show();
	TeikadeFile file = TeikadeFile.create(dialog.getDirectory() + File.separator + dialog.getFile());
	try {
	    CodeConverter.writeBytes(file, textArea.getText());
	} catch (IOException e) {
	    // we should do something, but what?
	}
    }

    public void start(String command) throws IOException {
	setCommand(command);
	start();
    }

    public void start() throws IOException {
	if (isRunning()) stop();
	Runtime runtime = Runtime.getRuntime();
	//process = runtime.exec(command, getEnvironment());
	process = runtime.exec(command);
	in = new DataInputStream(process.getInputStream());
	out = new PrintStream(process.getOutputStream());
	err = new DataInputStream(process.getErrorStream());
	try {
	    if (readThread != null) {
		    //readThread.stop();
		    ThreadUtil.Stop(readThread);
	    }
	} catch (RuntimeException e) {
	}
	readThread = new Thread(this,this.getClass().getName()+"-Read");
	readThread.start();
	try {
	    if (errorThread != null) {
		    //errorThread.stop();
		    ThreadUtil.Stop(errorThread);
	    }
	} catch (RuntimeException e) {
	}
	errorThread = new Thread(this,this.getClass().getName()+"-Error");
	errorThread.start();
    }

    public void stop() {
	try {
	    in.close();
	    out.close();
	    err.close();
	} catch (IOException e) {
	    // should not complain for the process being killed
	}
	if (isRunning()) {
	    process.destroy();
	    process = null;
	}
    }

    public void setCommand(String commandLine) {
	command = commandLine;
	setTitle(commandLine);
    }

    public boolean shouldEcho() {
	return echoOn;
    }

    public void echo(boolean onoff) {
	echoOn = onoff;
    }

    public boolean isRunning() {
	return readThread != null;
    }

    private String getEnvironment()[] {
	int size = environments.size();
	String envp[] = new String[size];
	Enumeration k = environments.keys();
	Enumeration e = environments.elements();
	for (int i = 0; i < size; i++) {
	    String s1 = k.nextElement().toString();
	    String s2 = e.nextElement().toString();
	    envp[i] = s1 + "=" + s2;
	}
	return envp;
    }

    /**
     * Main routine for independent use.
     */
    public static void main(String args[]) throws IOException {
	Terminal t = new Terminal();
	t.resize(500, 500);
	t.show();
	if (args.length != 0) {
	    StringBuffer buf = new StringBuffer(args[0]);
	    for (int i = 1; i < args.length; i++) {
		buf.append(' ').append(args[i]);
	    }
	    t.start(buf.toString());
	}
	try {
	    while (t.readThread == null) Thread.sleep(100);
	    t.readThread.join();
	} catch (InterruptedException e) {}
	System.exit(0);
    }
}
